try:
    from RNA import RNA
    from RNA.RNA import *
except Exception as E:
    raise ImportError('ViennaRNA / RNAlib not found')

ViennaRNA = RNA
viennarna = RNA
RNA       = RNA
rna       = RNA

def main():
    return RNA

if __name__ == '__main__':
    main()



__license__ = '''
    Disclaimer and Copyright

The programs, library and source code of the Vienna RNA Package are free
software. They are distributed in the hope that they will be useful
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

Permission is granted for research, educational, and commercial use
and modification so long as 1) the package and any derived works are not
redistributed for any fee, other than media costs, 2) proper credit is
given to the authors and the Institute for Theoretical Chemistry of the
University of Vienna.

If you want to include this software in a commercial product, please contact
the authors.

Note that the file ./src/ViennaRNA/plotting/naview.c has its own copyright attached.
'''

__doc__ = '''
A library for the prediction and comparison of RNA secondary structures.

Amongst other things, our implementations allow you to:

- predict minimum free energy secondary structures
- calculate the partition function for the ensemble of structures
- compute various equilibrium probabilities
- calculate suboptimal structures in a given energy range
- compute local structures in long sequences
- predict consensus secondary structures from a multiple sequence alignment
- predict melting curves
- search for sequences folding into a given structure
- compare two secondary structures 
- predict interactions between multiple RNA molecules
'''

__version__ = '2.7.2'

__author__ = 'Ronny Lorenz <rna@tbi.univie.ac.at>'

# Try loading RNA module by default and export all symbols to
# package namespace
try:
    from RNA.RNA import *
except Exception as E:
    raise ImportError('Failed to load ViennaRNA RNAlib Python wrapper')




import unittest

if __name__ == '__main__':
    from py_include import taprunner
    import RNApath
    RNApath.addSwigInterfacePath()

import RNA


seq1          = "CGCAGGGAUACCCGCG"
struct1       = "(((.(((...))))))"
seq1Dimer     = "CGCAGGGA&ACCCGCG"
struct1Dimer  = "(((.(((..))))))"
struct11      = "(((.((.....)))))"
##         1234567890
struct1_pt    = [len(struct1),16,15,14,0,13,12,11,0,0,0,7,6,5,3,2,1]


class mfe_eval_functionTest(unittest.TestCase):

    def test_mfe(self):
        """MFE prediction - single sequence"""
        fc= RNA.fold_compound(seq1)
        (ss,mfe) = fc.mfe()
        print(ss, "[ %6.2f ]" % mfe)
        self.assertEqual(ss,struct1)


    def test_mfe_Dimer(self):
        """MFE prediction - dimer"""
        fc=RNA.fold_compound(seq1Dimer)
        (ss,mfe) = fc.mfe_dimer()
        print(ss, "[ %6.2f ]" % mfe)
        self.assertEqual(ss,struct1Dimer)


    def test_eval_structure(self):
        """Structure energy evaluation - dot-bracket string"""
        fc = RNA.fold_compound(seq1)
        energy= fc.eval_structure(struct1)
        self.assertEqual("%6.2f" % energy, "%6.2f" % -5.60)
        print("\n", struct1, "[ %6.2f ]" % energy)


    def test_eval_structure_pt(self):
        """Structure energy evaluation - pair table"""
        fc=RNA.fold_compound(seq1)
        energy= fc.eval_structure_pt(struct1_pt) /100.; #/100 for dcal

        self.assertEqual("%6.2f" % energy, "%6.2f" % -5.60)
        print( struct1, "[ %6.2f ]" % energy)


    # Testing with filehandler and with stdout
    def test_eval_structure_verbose(self):
        """Structure energy evaluation - dot-bracket string - verbose output"""
        fc = RNA.fold_compound(seq1)
        filename= "test_RNA-mfe_eval.py.out"
        try:
            with open(filename, 'w') as f:
                print(filename ," is opened for writing")
                energy = fc.eval_structure_verbose(struct1, f)
                energy2 = fc.eval_structure_verbose(struct1)

                self.assertEqual("%6.2f" % energy, "%6.2f" % -5.60)
                print( struct1, "[ %6.2f ]" % energy)
        except IOError:
            print("Could not open ", filename)


    def test_eval_structure_pt_verbose(self):
        """Structure energy evaluation - pair table - verbose output"""
        filename= "test_RNA-mfe_eval.py.out"
        try:
            with open(filename, 'w') as f:
                print(filename, " is opened for writing")
                fc=RNA.fold_compound(seq1)

                energy = fc.eval_structure_pt_verbose(struct1_pt, f)/100.;  # / 100 for dcal
                energy2 = fc.eval_structure_pt_verbose(struct1_pt)/100.;  # / 100 for dcal

                self.assertEqual("%6.2f" % energy, "%6.2f" % -5.6)
                print( struct1, "[ %6.2f ]" % energy)

        except IOError:
            print("Could not open ", filename)


    def test_eval_covar_structure(self):
        """Structure energy evaluation - Covariance energy contribution"""
        s1="CCCCAAAACGGG"
        s2="CCCGAAAAGGGG"
        s3="CCCCAAAAGGGG"
        ali = [s1,s2,s3]
        covarStructure = "((((....))))"

        fc = RNA.fold_compound(ali)
        pseudoEScore=fc.eval_covar_structure(covarStructure)
        print(covarStructure, "[ %6.2f ]" % pseudoEScore)
        self.assertTrue(pseudoEScore)


    def test_eval_loop_pt(self):
        """Loop energy evaluation"""
        fc= RNA.fold_compound(seq1)
        energy= fc.eval_loop_pt(6, struct1_pt) / 100.; #/100 for dcal
        print("[ %6.2f ]" % energy)
        self.assertEqual("%6.2f" % energy,"%6.2f" % -3.3)


    def test_eval_move_del(self):
        """Move energy evaluation - base pair removal"""
        fc = RNA.fold_compound(seq1)
        energy = fc.eval_move(struct1, -7, -11);  # remove basepair (7,11) ,  energy change should be 2.10
        self.assertEqual("%6.2f" % energy, "%6.2f" % 2.10)
        print("\n", struct1, " moveset (-7,-11) --> [ %6.2f ]" % energy)


    def test_eval_move_ins(self):
        """Move energy evaluation - base pair insertion"""
        fc = RNA.fold_compound(seq1)
        energy = fc.eval_move(struct11, 7, 11);  # add basepair (7,11) ,  energy change should be -2.10
        self.assertEqual("%6.2f" % energy, "%6.2f" % -2.10)
        print("\n", struct11, " moveset (7,11) --> [ %6.2f ]" % energy)


    def test_eval_move_pt_del(self):
        """Move energy evaluation - base pair removal - pair table"""
        fc = RNA.fold_compound(seq1)
        energy = fc.eval_move_pt(struct1_pt, -7, -11) / 100.;  # remove basepair (7,11) ,  energy change should be 2.10
        self.assertEqual("%6.2f" % energy, "%6.2f" % 2.10)
        print("\n", struct1, " moveset (-7,-11) --> [ %6.2f ]" % energy)


if __name__ == '__main__':
    unittest.main(testRunner=taprunner.TAPTestRunner())



import unittest

if __name__ == '__main__':
    from py_include import taprunner
    import RNApath
    RNApath.addSwigInterfacePath()

import RNA

seq = "CCCGCCCCACCACUCCAGCAGACCUUGCCCCUUGUGAGCUGGAUAGACUUGGGAUGGGGAGGGAGGGAGUUUUGUCUGUCUCCCUCCCCUCUCAGAACAUACUGAUUGGGAGGUGCG"

def t(x, i, n):
    """
    Circular rotation of sequence position x by i positions for sequence of length n
    """
    return ((x + i) - 1) % n + 1

def c(x, y, i, n):
    """
    Circular rotation of base pair positions (x,y) by i positions for sequence of length m
    """
    xx, yy = t(x, i, n), t(y, i, n)
    return (xx, yy) if yy > xx else (yy, xx)


def rotate_structure(db, i):
    """
    Circular rotation of a dot-bracket structure by i positions
    
    Simply rotating the db string like:
    
    db = db[n-i:] + db[:n - i]

    doesn't work sinze some base pairs may end up as ) ( pair
    instead of ( ). So, we transform the structure at pair
    table level and convert it back to dot-bracket for later
    comparison.
    
    In addition, any non-dot-bracket characters will be re-inserted
    as they are not present at the pair table level
    """
    n = len(db)

    if (i % n) == 0:
        return str(db)

    pt_db = [ e for e in RNA.ptable(db) ]
    pt    = [ 0 for _ in range(n + 1) ]
    pt[0] = n

    for j, p in enumerate(pt_db[1:], start = 1):
        if p > j:
            ii, jj = c(j, p, i, n)
            pt[jj], pt[ii] = ii, jj

    ss = list(RNA.db_from_ptable(pt))

    # re-insert any special characters, such as gquads
    for j, cc in enumerate(list(db), start = 1):
        if cc not in ["(", ")", "."]:
            ss[t(j, i, n) - 1] = cc

    return "".join(ss)


def rotate_pair_probs(plist, i, n, ptype = RNA.PLIST_TYPE_BASEPAIR):
    """
    Circular rotation of a pair probability list by i positions
    """
    return sorted([ (*c(p.i, p.j, i, n), f"{p.p:.10f}") for p in plist if p.type == ptype ])


class CircMFETest(unittest.TestCase):
    def test_mfe(self):
        """Circular RNA MFE and structure"""

        md        = RNA.md()
        md.circ   = 1
        n         = len(seq)
        fc        = RNA.fold_compound(seq, md)
        ss0, mfe0 = fc.mfe()

        for i in range(1, n):
            # rotate sequence by i
            s = seq[i:] + seq[:i]

            fc = RNA.fold_compound(s, md)
            ss, mfe = fc.mfe()

            # rotate structure by i
            ss = rotate_structure(ss, i)

            self.assertEqual(mfe, mfe0)
            self.assertEqual(ss, ss0)

    def test_mfe_gquad(self):
        """Circular RNA MFE and structure (G-Quadruplexes)"""

        md        = RNA.md()
        md.circ   = 1
        md.gquad  = 1
        n         = len(seq)
        fc        = RNA.fold_compound(seq, md)
        ss0, mfe0 = fc.mfe()

        for i in range(1, n):
            # rotate sequence by i
            s = seq[i:] + seq[:i]

            fc = RNA.fold_compound(s, md)
            ss, mfe = fc.mfe()

            # rotate structure by i
            ss = rotate_structure(ss, i)

            self.assertEqual(mfe, mfe0)
            self.assertEqual(ss, ss0)


class CircPartFunTest(unittest.TestCase):
    def test_pf(self):
        """Circular RNA partition function and pair probabilities"""
        cutoff    = 1e-5

        md        = RNA.md()
        md.circ   = 1
        n         = len(seq)
        fc        = RNA.fold_compound(seq, md)
        ss0, mfe0 = fc.mfe()
        fc.exp_params_rescale(mfe0)
        pp0, dG0  = fc.pf()
        pairs0    = rotate_pair_probs(fc.plist_from_probs(cutoff), 0, n)

        for i in range(1, n):
            # rotate sequence by i
            s = seq[i:] + seq[:i]

            fc      = RNA.fold_compound(s, md)
            ss, mfe = fc.mfe()
            fc.exp_params_rescale(mfe)
            pp, dG  = fc.pf()
            pairs   = rotate_pair_probs(fc.plist_from_probs(cutoff), i, n)

            # rotate structure by i
            self.assertEqual(dG, dG0)

            # rotate pairing propensity string
            # pp = pp[n-i:] + pp[:n - i]
            # this doesn't always work, and for now, we don't have
            # a sane way to transform similar to MFE structure above
            #self.assertEqual(pp, pp0)

            # compare base pair probabilities
            self.assertEqual(pairs, pairs0)

    def test_pf_gquad(self):
        """Circular RNA partition function and pair probabilities (G-Quadruplexes)"""
        cutoff    = 1e-5

        md        = RNA.md()
        md.circ   = 1
        md.gquad  = 1
        n         = len(seq)
        fc        = RNA.fold_compound(seq, md)
        ss0, mfe0 = fc.mfe()
        fc.exp_params_rescale(mfe0)
        pp0, dG0  = fc.pf()
        pairs0    = rotate_pair_probs(fc.plist_from_probs(cutoff), 0, n)
        qgs0      = rotate_pair_probs(fc.plist_from_probs(cutoff), 0, n, RNA.PLIST_TYPE_GQUAD)

        for i in range(1, n):
            # rotate sequence by i
            s = seq[i:] + seq[:i]

            fc      = RNA.fold_compound(s, md)
            ss, mfe = fc.mfe()
            fc.exp_params_rescale(mfe)
            pp, dG  = fc.pf()
            pairs   = rotate_pair_probs(fc.plist_from_probs(cutoff), i, n)
            qgs     = rotate_pair_probs(fc.plist_from_probs(cutoff), i, n, RNA.PLIST_TYPE_GQUAD)

            # rotate structure by i
            self.assertEqual(dG, dG0)

            # rotate pairing propensity string
            # pp = pp[n-i:] + pp[:n - i]
            # this doesn't always work, and for now, we don't have
            # a sane way to transform similar to MFE structure above
            #self.assertEqual(pp, pp0)

            # compare base pair probabilities
            self.assertEqual(pairs, pairs0)

            # compare gquad probabilities
            self.assertEqual(qgs, qgs0)


if __name__ == '__main__':
    unittest.main(testRunner=taprunner.TAPTestRunner())




import unittest
from struct import *
import locale
import json
import re

locale.setlocale(locale.LC_ALL, 'C')

if __name__ == '__main__':
    from py_include import taprunner
    import RNApath
    RNApath.addSwigInterfacePath()

import RNA


def extract_duplex_data(duplex, data, one_letter_code, fallback):
    # use predicted duplex stability from experimenters if available
    e  = data['dG37_p'] if 'dG37_p' in data else data['dG37']
    s1 = duplex[:data['length1']]
    s2 = duplex[data['length1']:]
    s2 = s2[::-1]
    mpos = [m.start() + 1 for m in re.finditer(one_letter_code, s1 + s2)]
    seq = s1 + "&" + s2
    seq = seq.replace(one_letter_code, fallback)

    return e, seq, mpos


class ModifiedBaseTests(unittest.TestCase):
    """Some tests for modified bases"""

    def test_pseudouridine_duplexes(self):
        """Pseudouridine duplex energies"""
        params = json.loads(RNA.parameter_set_rna_mod_pseudouridine_parameters)
        duplexes = params['modified_base']['duplexes']
        code     = params['modified_base']['one_letter_code']
        fallback = params['modified_base']['fallback']
        for k,v in duplexes.items():
            e, seq, mpos = extract_duplex_data(k, v, code, fallback)
            # prepare computation
            fc = RNA.fold_compound(seq)
            fc.sc_mod_pseudouridine(mpos)
            e_diff = e - fc.mfe()[1]
            print(seq, mpos, e, e_diff, e - e_diff)
            self.assertTrue(abs(e_diff) < 0.2)


    def test_inosine_duplexes(self):
        """Inosine duplex energies"""
        params = json.loads(RNA.parameter_set_rna_mod_inosine_parameters)
        duplexes = params['modified_base']['duplexes']
        code     = params['modified_base']['one_letter_code']
        fallback = params['modified_base']['fallback']
        for k,v in duplexes.items():
            e, seq, mpos = extract_duplex_data(k, v, code, fallback)
            # prepare computation
            fc = RNA.fold_compound(seq)
            fc.sc_mod_inosine(mpos)
            e_diff = e - fc.mfe()[1]
            print(seq, mpos, e, e_diff, e - e_diff)
            self.assertTrue(abs(e_diff) < 0.15)


if __name__ == '__main__':
    unittest.main(testRunner=taprunner.TAPTestRunner())




import os, pytest

def pytest_addoption(parser):
    parser.addoption(
        "--datadir", action="store", default="tests/data", help="location of data dir"
    )

def pytest_sessionstart(session):
    os.environ['VRNA_TEST_DATA'] = session.config.getoption("--datadir")



import unittest

if __name__ == '__main__':
    from py_include import taprunner
    import RNApath
    RNApath.addSwigInterfacePath()

import RNA


seq_con     = "CCCAAAAGGGCCCAAAAGGG"
short_seq   = "UGGGAAUAGUCUCUUCCGAGUCUCGCGGGCGACGGGCGAUCUUCGAAAGUGGAAUCCGUA"
seq_long    = "AUUUCCACUAGAGAAGGUCUAGAGUGUUUGUCGUUUGUCAGAAGUCCCUAUUCCAGGUACGAACACGGUGGAUAUGUUCGACGACAGGAUCGGCGCACUACGUUGGUAUCAUGUCCUCCGUCCUAACAAUUAUACAUCGAGAGGCAAAAUUUCUAAUCCGGGGUCAGUGAGCAUUGCCAUUUUAUAACUCGUGAUCUCUC"
str_con     = "..........(((....)))"
str_con_def = "(((....)))(((....)))"

def mfe_window_callback(start, end, structure, energy, data=None):
    data.append({ 'structure': structure, 'start': start, 'end' : end, 'energy' : energy})


class constraintsTest(unittest.TestCase):

    def test_sc_set_up(self):
        """Soft constraints - unpaired nucleotides"""
        seq_sc  =      "CCCAAAAGGG"
        fc = RNA.fold_compound(seq_sc)
        (ss,mfe) = fc.mfe()
        print(ss, "[ %6.2f ]" % mfe)
        self.assertEqual(ss,"(((....)))")

        fc.sc_init()

        m= [0.0,-5.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0];  #E 1 0 1 -5 ,  position 1 gets -5 if unpaired , vector starts with 0 and not 1

        fc.sc_set_up(m)
        (ss,mfeNew) = fc.mfe()
        print(ss, "[ %6.2f ]" % mfeNew)
        self.assertEqual("%6.2f" %mfeNew,"%6.2f" % -5.70)


    def test_sc_set_bp(self):
        """Soft constraints - base pairs"""
        #add energy of -5 to basepair 1-9 if formed, prefed structure should now be ((.....))., with a energy of -4.90, #matrix is also beginning with position 0
        m = [[0 for x in range(11)] for y in range(11)]
        m[1][9] = -5.0; # base 1-9 should get -5.0 if basepair
        m[9][1] = -5.0; # base 1-9 should get -5.0 if basepair

        seq_sc = "CCCAAAAGGG"
        fc = RNA.fold_compound(seq_sc)
        fc.sc_set_bp(m)
        (ss,mfeNew) = fc.mfe()
        print(ss, "[ %6.2f ]" % mfeNew)
        self.assertEqual("%6.2f" %mfeNew,"%6.2f" % -4.90)


    def test_sc_shift(self):
        """
        Soft constraints - shift energy landscape

        Compute partition function and base pair probabilities both, constrained
        and unconstrained, where the constraint simply shifts the free energy base
        line by -1 kcal/mol per nucleotide.
        When comparing both results, equilibrium probabilities must not have changed,
        except for free energy of the ensemble!
        """
        fc = RNA.fold_compound(short_seq)
        # unconstrained partition function
        ss, dG = fc.pf()

        bpp = fc.bpp()

        # add constraints
        for i in range(1, len(short_seq) + 1):
          fc.sc_add_up(i, -1.0)

        for i in range(1, len(short_seq)):
            for j in range(i + 1, len(short_seq) + 1):
                fc.sc_add_bp(i, j, -2)

        # constrained partition function
        ss2, dG2 = fc.pf()
        bpp2 = fc.bpp()

        # check if ensemble free energies are actually different
        self.assertTrue(dG != dG2)

        # check nucleotide pairing propensities
        self.assertEqual(ss, ss2)

        # check pairing probabilities
        for i in range(1, len(short_seq)):
            for j in range(i + 1, len(short_seq) + 1):
                self.assertEqual("%1.8f" % bpp[i][j], "%1.8f" % bpp2[i][j])


class constraintsMFEWindowTest(unittest.TestCase):
    def test_sc_mfe_window_add_bp(self):
        """Soft constraints - base pairs sliding window MFE"""
        fc = RNA.fold_compound(seq_long, None, RNA.OPTION_WINDOW)

        # add twice -10.0 kcal/mol on base pair (55,60)
        fc.sc_add_bp(55, 60, -10.0, RNA.OPTION_WINDOW)
        fc.sc_add_bp(55, 60, -10.0, RNA.OPTION_WINDOW)

        # allow pair (55,60) to actually form (might be non-canonical)
        fc.hc_add_bp(55, 60, RNA.CONSTRAINT_CONTEXT_NO_REMOVE | RNA.CONSTRAINT_CONTEXT_ALL_LOOPS)

        data = []
        mfe = fc.mfe_window_cb(mfe_window_callback, data)

        for hit in data:
            if (hit['start'] <= 55) and (hit['end'] >= 60):
                # compose actual dot bracket string (including potential 5' dangle nucleotide
                if hit['start'] > 1:
                    d = 1
                    ss = "." + hit['structure']
                else:
                    d = 0
                    ss = hit['structure']

                # get corresponding subsequence
                s = seq_long[hit['start'] - 1 - d: hit['end']]

                # re-evaluate free energy of subsequence/hit
                e = RNA.energy_of_struct(s, ss)

                # energy difference between both must be -20.0 kcal/mol (if the constrained base pair is present)
                self.assertEqual("%6.2f" % hit['energy'], "%6.2f" % (e - 20.0))


    def test_sc_mfe_window_add_up(self):
        """Soft constraints - unpaired nucleotides sliding window MFE"""
        fc = RNA.fold_compound(seq_long, None, RNA.OPTION_WINDOW)

        # add twice -5.0 kcal/mol per unpaired nucleotide in segment [55,60]
        for i in range(50, 61):
            fc.sc_add_up(i, -5.0, RNA.OPTION_WINDOW)

        # force segment [50,60] to stay unpaired
        for i in range(50,61):
            fc.hc_add_up(i, RNA.CONSTRAINT_CONTEXT_ALL_LOOPS)

        data = []
        mfe = fc.mfe_window_cb(mfe_window_callback, data)

        for hit in data:
            if (hit['start'] <= 55) and (hit['end'] >= 60):
                # count number of unpiared nucleotides with bonus
                c = 0
                for i in range(50, 61):
                    if hit['structure'][i - hit['start']] == '.':
                        c = c + 1

                # compose actual dot bracket string (including potential 5' dangle nucleotide
                if hit['start'] > 1:
                    d = 1
                    ss = "." + hit['structure']
                else:
                    d = 0
                    ss = hit['structure']

                # get corresponding subsequence
                s = seq_long[hit['start'] - 1 - d: hit['end']]

                # re-evaluate free energy of subsequence/hit
                e = RNA.energy_of_struct(s, ss)

                # energy difference between both must be c * -5.0 kcal/mol
                self.assertEqual("%6.2f" % hit['energy'], "%6.2f" % (e + (c * -5.0)))



if __name__ == '__main__':
    unittest.main(testRunner=taprunner.TAPTestRunner())



import unittest

if __name__ == '__main__':
    from py_include import taprunner
    import RNApath
    RNApath.addSwigInterfacePath()

import RNA


seq1 = "AUUUCCACUAGAGAAGGUCUAGAGUGUUUGUCGUUUGUCAGAAGUCCCUAUUCC"
struct1 = ".(((..((.((((((((((....).)))).)).))))).)))(((....))).."
seq2 = "GGGGAAAACCCC"
struct2 = "((((....))))"
struct3 = "(((((..)))))"

c = { 'what' : "theheck" }

def MaximumMatching(i,j,k,l,d,data=None):
    e = 0

    if d == RNA.DECOMP_PAIR_HP:
        e = e - data['fc'].eval_hp_loop(i,j)
        e = e - 100

    elif d == RNA.DECOMP_PAIR_IL:
        e = e - data['fc'].eval_int_loop(i, j, k, l)
        e = e - 100

    elif d == RNA.DECOMP_PAIR_ML:
        u = j - i - 1 - (k - l + 1)
        e = e - data['params'].MLclosing
        e = e - data['params'].MLintern[0]
        e = e - u * data['params'].MLbase

        e = e - 100

    elif d == RNA.DECOMP_ML_ML_STEM:
        u = j - i + 1 - (j - l + 1) - (k - i + 1)
        e = e - data['params'].MLintern[0]
        e = e - u * data['params'].MLbase

    elif d == RNA.DECOMP_ML_STEM:
        u = j - i + 1 - (k - l + 1)
        e = e - data['params'].MLintern[0]
        e = e - u * data['params'].MLbase

    elif d == RNA.DECOMP_ML_ML:
        u = j - i + 1 - (k - l + 1)
        e = e - u * data['params'].MLbase

    elif d == RNA.DECOMP_ML_UP:
        u = j - i + 1
        e = e - u * data['params'].MLbase

    elif d == RNA.DECOMP_EXT_STEM:
        e = e - data['fc'].eval_ext_stem(k, l)

    elif d == RNA.DECOMP_EXT_STEM_EXT:
        e = e - data['fc'].eval_ext_stem(i, k)

    elif d == RNA.DECOMP_EXT_EXT_STEM:
        e = e - data['fc'].eval_ext_stem(l, j)

    elif d == RNA.DECOMP_EXT_EXT_STEM1:
        e = e - data['fc'].eval_ext_stem(l, j-1)

    return e


def blubb(i,j,k,l,d,data=None):
    if d == RNA.DECOMP_PAIR_HP:
        """
        Add -10 kcal/mol to any hairpin
        """
        return -1000

    return 0


"""
The backtracking callback must return a list of base pairs
Here, the base pairs may be given in one of the three ways
shown below:
"""

"""
1. We create a list of dictionaries with 'i' and 'j'
keys that specify the coordinates of the base pair (i,j)
"""
def bt_hp_dict(i,j,k,l,d,data=None):
    if d == RNA.DECOMP_PAIR_HP:
        return [{ 'i' : i+1, 'j' : j-1 }]

    return None


"""
2. We create a list of tuples (i,j)
"""
def bt_hp_tuples(i,j,k,l,d,data=None):
    if d == RNA.DECOMP_PAIR_HP:
        return [(i+1, j-1)]

    return None


"""
3. We create a list of RNA::basepair objects
"""
def bt_hp_basepair(i,j,k,l,d,data=None):
    if d == RNA.DECOMP_PAIR_HP:
        bp = RNA.basepair()
        bp.i = i+1
        bp.j = j-1

        return [ bp ]

    return None


class mfe_eval_functionTest(unittest.TestCase):

    def test_maximum_matching(self):
        """Revert MFE to MaximumMatching"""
        md = RNA.md(dangles=0)

        mm_data = { 'fc': RNA.fold_compound(seq1, md),
                    'params': RNA.param(md)
                  }
        fc = RNA.fold_compound(seq1, md)
        fc.sc_add_data(mm_data, None)
        fc.sc_add_f(MaximumMatching)
        (s, mm) = fc.mfe()
        print("%s\n%s (max BPs: %d)" % (seq1, s, -mm))
        self.assertEqual(s, struct1)
        self.assertTrue(-mm == 18)

        mm_data = { 'fc': RNA.fold_compound(seq2, md),
                    'params': RNA.param(md)
                  }
        fc = RNA.fold_compound(seq2, md)
        fc.sc_add_data(mm_data, None)
        fc.sc_add_f(MaximumMatching)
        (s, mm) = fc.mfe()
        print("%s\n%s (max BPs: %d)" %  (seq2, s, -mm))
        self.assertEqual(s, struct2)
        self.assertTrue(-mm == 4)


    def test_backtrack_hp_dict(self):
        """Hairpin backtracking (Dictionary)"""
        fc = RNA.fold_compound(seq2)
        fc.sc_add_bt(bt_hp_dict)
        (s, mfe) = fc.mfe()
        print("%s %6.2f" %  (s, mfe))
        self.assertEqual(s, struct3)


    def test_backtrack_hp_tuples(self):
        """Hairpin backtracking (Tuples)"""
        fc = RNA.fold_compound(seq2)
        fc.sc_add_bt(bt_hp_tuples)
        (s, mfe) = fc.mfe()
        print("%s %6.2f" %  (s, mfe))
        self.assertEqual(s, struct3)


    def test_backtrack_hp_basepair(self):
        """Hairpin backtracking (basepair)"""
        fc = RNA.fold_compound(seq2)
        fc.sc_add_bt(bt_hp_basepair)
        (s, mfe) = fc.mfe()
        print("%s %6.2f" %  (s, mfe))
        self.assertEqual(s, struct3)


if __name__ == '__main__':
    unittest.main(testRunner=taprunner.TAPTestRunner())



import unittest

if __name__ == '__main__':
    from py_include import taprunner
    import RNApath
    RNApath.addSwigInterfacePath()

import RNA


motif_seq_theo = "GAUACCAG&CCCUUGGCAGC"
motif_str_theo = "(...((((&)...)))...)"
e_binding_theo = -9.22

str_hp    = "((((...((((.....(.(.....).).))))...))))...((.((((((((((.((((((.((.((((....)))).))..)))))).)))))))))))).."
e_hp      = -35.60
str_theo  = "((((...((((((((......)))))...)))...))))...((.((((((((((.((((((.((.((((....)))).))..)))))).)))))))))))).."
e_theo    = -33.82

class constraintsTest(unittest.TestCase):

    def test_sc_add_hi_motif(self):
        """Soft constraints - Motif - Hairpin"""
        fc = RNA.fold_compound("GGUGAUACCAGAUUUCGCGAAAAAUCCCUUGGCAGCACCUCGCACAUCUUGUUGUCUGAUUAUUGAUUUUUCGCGAAACCAUUUGAUCAUAUGACAAGAUUGAG")
        fc.sc_add_hi_motif("GAAAAAU", "(.....)", -19)
        (ss, mfe) = fc.mfe()
        print("%s [ %6.2f ]" % (ss, mfe))


    def test_theophylline_ligand_binding_interface(self):
        """Soft constraints - Motif - Theophylline aptamer"""
        fc  = RNA.fold_compound("GGUGAUACCAGAUUUCGCGAAAAAUCCCUUGGCAGCACCUCGCACAUCUUGUUGUCUGAUUAUUGAUUUUUCGCGAAACCAUUUGAUCAUAUGACAAGAUUGAG")
        ret = fc.sc_add_hi_motif(motif_seq_theo, motif_str_theo, e_binding_theo)
        (ss, mfe) = fc.mfe()
        print("%s [ %6.2f ]" % (ss, mfe))
        self.assertEqual(ret,1)
        self.assertEqual(ss, str_theo)
        self.assertAlmostEqual(mfe, e_theo, delta=1e-3)


if __name__ == '__main__':
    unittest.main(testRunner=taprunner.TAPTestRunner())



import unittest
import sys

if __name__ == '__main__':
    from py_include import taprunner
    import RNApath
    RNApath.addSwigInterfacePath()

import RNA


sequence = "UGGGAAUAGUCUCUUCCGAGUCUCGCGGGCGACGGGCGAUCUUCGAAAGUGGAAUCCGUACUUAUACCGCCUGUGCGGACUACUAUCCUGACCACAUAGU"

def store_structure(s, data):
    if s:
        data.append(s)


def prepare_fc():
    md          = RNA.md()
    md.uniq_ML  = 1
    fc          = RNA.fold_compound(sequence, md)
    (ss, mfe)   = fc.mfe()

    fc.exp_params_rescale(mfe)
    fc.pf()

    return fc

class GeneralTests(unittest.TestCase):
    def test_pbacktrack5(self):
        """Boltzmann sampling (single sub-structure, a.k.a. pbacktrack5)"""
        fc = prepare_fc()

        s  = fc.pbacktrack5(10)
        self.assertEqual(len(s), 10)

        s  = fc.pbacktrack5(50)
        self.assertEqual(len(s), 50)

    def test_pbacktrack5_multi(self):
        """Boltzmann sampling (multiple sub-structures, a.k.a. pbacktrack5_num)"""
        fc = prepare_fc()

        ss  = fc.pbacktrack5(20, 10)
        self.assertEqual(len(ss), 20)
        for s in ss:
            self.assertEqual(len(s), 10)

        ss  = fc.pbacktrack5(100, 50)
        self.assertEqual(len(ss), 100)
        for s in ss:
            self.assertEqual(len(s), 50)

    def test_pbacktrack(self):
        """Boltzmann sampling (single structure, a.k.a. pbacktrack)"""
        fc = prepare_fc()

        s  = fc.pbacktrack()
        self.assertEqual(len(s), len(sequence))

    def test_pbacktrack_multi(self):
        """Boltzmann sampling (multiple structures, a.k.a. pbacktrack_num)"""
        fc = prepare_fc()

        ss  = fc.pbacktrack(100)
        self.assertEqual(len(ss), 100)
        for s in ss:
            self.assertEqual(len(s), len(sequence))

    @unittest.skipIf(sys.platform.startswith("win"), "May fail under Windows")
    def test_pbacktrack_nr(self):
        """Boltzmann sampling - non redundant"""
        try:
            import pytest
            if sys.platform.startswith("win"):
                pytest.skip("May fail under Windows")
        except:
            pass

        fc = prepare_fc()

        ss  = fc.pbacktrack(100, RNA.PBACKTRACK_NON_REDUNDANT)
        self.assertEqual(len(ss), 100)
        for s in ss:
            self.assertEqual(len(s), len(sequence))
        # check for uniqueness, i.e. no duplicates
        sss = list(set(ss))
        self.assertEqual(len(ss), len(sss))

    @unittest.skipIf(sys.platform.startswith("win"), "May fail under Windows")
    def test_pbacktrack_nr_resume(self):
        """Boltzmann sampling - non redundant resume"""
        try:
            import pytest
            if sys.platform.startswith("win"):
                pytest.skip("May fail under Windows")
        except:
            pass

        num_samples = 500
        iterations  = 15
        fc      = prepare_fc()
        d       = None
        s_list  = list()
        for _ in range(0, iterations):
            d, ss   = fc.pbacktrack(num_samples, d, RNA.PBACKTRACK_NON_REDUNDANT)
            s_list  = s_list + list(ss)

        self.assertEqual(len(s_list), iterations * num_samples)

        for s in s_list:
            self.assertEqual(len(s), len(sequence))

        # check for uniqueness, i.e. no duplicates
        sss = list(set(s_list))
        self.assertEqual(len(s_list), len(sss))

    def test_pbacktrack5_cb(self):
        """Boltzmann sampling (multiple sub-structures, a.k.a. pbacktrack5_cb)"""
        fc = prepare_fc()
        ss = list()
        i = fc.pbacktrack5(20, 10, store_structure, ss)
        self.assertEqual(i, 20)
        self.assertEqual(len(ss), 20)
        for s in ss:
            self.assertEqual(len(s), 10)

        ss = list()
        i = fc.pbacktrack5(100, 50, store_structure, ss)
        self.assertEqual(i, 100)
        self.assertEqual(len(ss), 100)
        for s in ss:
            self.assertEqual(len(s), 50)

    def test_pbacktrack_cb(self):
        """Boltzmann sampling (multiple structures, a.k.a. pbacktrack_cb)"""
        fc = prepare_fc()
        ss = list()
        i = fc.pbacktrack(100, store_structure, ss)
        self.assertEqual(i, 100)
        self.assertEqual(len(ss), 100)
        for s in ss:
            self.assertEqual(len(s), len(sequence))

    @unittest.skipIf(sys.platform.startswith("win"), "May fail under Windows")
    def test_pbacktrack_nr_cb(self):
        """Boltzmann sampling - non redundant - callback"""
        try:
            import pytest
            if sys.platform.startswith("win"):
                pytest.skip("May fail under Windows")
        except:
            pass

        fc = prepare_fc()
        ss = list()
        i = fc.pbacktrack(100, store_structure, ss, RNA.PBACKTRACK_NON_REDUNDANT)
        self.assertEqual(i, 100)
        self.assertEqual(len(ss), 100)
        for s in ss:
            self.assertEqual(len(s), len(sequence))

        # check for uniqueness, i.e. no duplicates
        sss = list(set(ss))
        self.assertEqual(len(ss), len(sss))

    @unittest.skipIf(sys.platform.startswith("win"), "May fail under Windows")
    def test_pbacktrack_nr_resume_cb(self):
        """Boltzmann sampling - non redundant resume - callback"""
        try:
            import pytest
            if sys.platform.startswith("win"):
                pytest.skip("May fail under Windows")
        except:
            pass

        num_samples = 500
        iterations  = 15
        fc = prepare_fc()
        ss = list()
        d  = None
        for _ in range(0, iterations):
            d, i = fc.pbacktrack(num_samples, store_structure, ss, d, RNA.PBACKTRACK_NON_REDUNDANT)
            self.assertEqual(i, num_samples)

        self.assertEqual(len(ss), iterations * num_samples)
        for s in ss:
            self.assertEqual(len(s), len(sequence))

        # check for uniqueness, i.e. no duplicates
        sss = list(set(ss))
        self.assertEqual(len(ss), len(sss))


if __name__ == '__main__':
    unittest.main(testRunner=taprunner.TAPTestRunner())



from setuptools.command.build_ext import build_ext
from setuptools.command.build_py import build_py
from setuptools import setup
from setuptools import find_packages
from setuptools import Extension

try:
    import configparser
except ImportError:
    import ConfigParser as configparser

import os
import glob
import sys
import platform
import re

from tempfile import mkstemp
from shutil import move, copymode
from os import fdopen, remove


vrna_flags = [
    'with-svm',
    'with-naview',
    'with-gsl',
    'with-mpfr',
    'with-openmp',
    'with-simd',
]

vrna_options = [
    ('svm_macro', None,
     "Preprocessor macro activating SVM support."),
    ('svm_libs', None,
     "Additional libraries required when linking with SVM support."),
    ('gsl_macro', None,
     "Preprocessor macro activating GSL support."),
    ('gsl_libs', None,
     "Additional libraries required when linking with GSL support."),
    ('mpfr_macro', None,
     "Preprocessor macro activating MPFR support."),
    ('mpfr_libs', None,
     "Additional libraries required when linking with MPFR support."),
    ('naview_macro', None,
     "Preprocessor macro activating NAVIEW layout support."),
    ('openmp_macro', None,
     "Preprocessor macro activating OpenMP support."),
    ('openmp_cflags', None,
     "OpenMP compiler/linker flags."),
]

vrna_simd_cflags = {
    'SSE41': '-msse4.1',
    'AVX512': '-mavx512f',
}

vrna_simd_files = {
    'SSE41' : [
        'src/ViennaRNA/utils/higher_order_functions_sse41.c',
    ],
    'AVX512' : [
        'src/ViennaRNA/utils/higher_order_functions_avx512.c',
    ],
}

# readme
THIS_DIRECTORY = os.path.abspath(os.path.dirname(__file__))
README_FILE = os.path.join(THIS_DIRECTORY, "README.md")
LONG_DESCRIPTION = ""

remove_sections = [
  "Table of Contents",
  "Executable Programs",
  "Configuration",
  "Energy Parameters"
]

header_pat = re.compile(r'^(\#+)\s*(.*)$')

with open(README_FILE, encoding="utf-8") as readme:
    within_section_remove = False
    section_level = 0
    for l in readme:
        m = header_pat.match(l)
        if m:
            if within_section_remove:
                if len(m.group(1)) <= section_level:
                    if m.group(2) in remove_sections:
                        section_level = len(m.group(1))
                        continue
                    else:
                        within_section_remove = False
                else:
                    continue
            elif m.group(2) in remove_sections:
                section_level = len(m.group(1))
                within_section_remove = True
                continue
        elif within_section_remove:
            continue

        LONG_DESCRIPTION += l


def comment_lines(file_path, patterns, comment_char = "//"):
    #Create temp file
    fh, abs_path = mkstemp()
    with fdopen(fh,'w') as new_file:
        with open(file_path) as old_file:
            for line in old_file:
                replaced = False
                for p in patterns:
                    if line.startswith(p):
                        new_file.write(comment_char + line)
                        replaced = True
                        break
                if not replaced:
                    new_file.write(line)
    #Copy the file permissions from the old file to the new file
    copymode(file_path, abs_path)
    #Remove original file
    remove(file_path)
    #Move new file
    move(abs_path, file_path)


class vrna_build_ext(build_ext):
    user_options = build_ext.user_options[:]
    user_options.extend(vrna_options)

    boolean_options = build_ext.boolean_options[:]
    boolean_options.extend(vrna_flags)

    def __init__(self, *args, **kwargs):
        build_ext.__init__(self, *args, **kwargs)

    def run(self):
        build_ext.run(self)

    def initialize_options(self):
        self.parse_options()
        build_ext.initialize_options(self)

    def compiler_is_msvc(self):
        return self.get_compiler_name().lower().startswith('msvc')

    def compiler_is_mingw(self):
        return self.get_compiler_name().lower().startswith('mingw')

    def get_compiler_name(self):
        """Return the name of the C compiler used to compile extensions.

        If a compiler was not explicitly set (on the command line, for
        example), fall back on the default compiler.
        """
        if self.compiler:
            # distutils doesn't keep the type of self.compiler uniform; we
            # compensate:
            if isinstance(self.compiler, str):
                name = self.compiler
            else:
                name = self.compiler.compiler_type
        else:
            name = get_default_compiler()
        return name

    def build_extensions(self):
        for ext in self.extensions:
            ext.define_macros += [('VRNA_VERSION', "\"2.7.2\"")]
            # we include config.h, so let's tell the source files about it
            if not self.compiler_is_msvc():
                ext.define_macros += [('HAVE_CONFIG_H', None)]

            ext.libraries += ['m']

            ext.define_macros += [('VRNA_LOG_NO_DEBUG_RNALIB', "1")]

            if sys.platform == 'darwin':
                # always disable OpenMP on Mac OS X (until we resolve a good way to activate it)
                self.with_openmp = False
                ext.extra_compile_args += ["-stdlib=libc++",
                  "-mmacosx-version-min=10.14",
                  "-std=c++1z"]  # c++17 for dlib 19.24

                # deactivate SIMD implementations for ARM builds
                if platform.machine() == "arm64" or "arm64" in os.getenv("ARCHFLAGS", ""):
                    self.with_simd = False
                    # collect all SIMD specific files
                    simd_files = [f for tmp in vrna_simd_files.values() for f in tmp]
                    ext.sources = [s for s in ext.sources if s not in simd_files]
                else:
                    ext.define_macros += [('VRNA_WITH_SIMD_AVX512', None)]
                    ext.define_macros += [('VRNA_WITH_SIMD_SSE41', None)]
            elif self.compiler_is_msvc():
                self.with_openmp = False
                ext.define_macros += [('VRNA_WITH_SIMD_AVX512', None)]
                ext.define_macros += [('VRNA_WITH_SIMD_SSE41', None)]
                # disable C11 features since they are supported by MSVC only in parts (if at all)
                ext.define_macros += [('VRNA_DISABLE_C11_FEATURES', None)]
                ext.sources.append("src/dlib-20.0/dlib/all/source.cpp")
            else:
                if (platform.machine() == "arm64" or "arm64" in os.getenv("ARCHFLAGS", "")) or \
                   (platform.machine() == "aarch64" or "aarch64" in os.getenv("ARCHFLAGS", "")):
                    self.with_simd = False
                    # collect all SIMD specific files
                    simd_files = [f for tmp in vrna_simd_files.values() for f in tmp]
                    ext.sources = [s for s in ext.sources if s not in simd_files]
                else:
                    ext.define_macros += [('VRNA_WITH_SIMD_AVX512', None)]
                    ext.define_macros += [('VRNA_WITH_SIMD_SSE41', None)]


            for sw in [f.replace('with-', '') for f in vrna_flags]:
                if getattr(self, 'with_' + sw):
                    if getattr(self, sw + '_macro', None):
                        ext.define_macros += [(getattr(self, sw + '_macro'), None)]
                    if getattr(self, sw + '_libs', None):
                        ext.libraries += [l for l in getattr(self, sw + '_libs').split()]
                    if getattr(self, sw + '_cflags', None):
                        ext.extra_compile_args += [f for f in getattr(self, sw + '_cflags').split()]
                        ext.extra_link_args += [f for f in getattr(self, sw + '_cflags').split()]

            if self.compiler_is_msvc():
                ext.libraries.remove("m")
                try:
                    ext.libraries.remove("stdc++")
                except:
                    pass


        build_ext.build_extensions(self)

    def build_extension(self, ext):
        # overwrite build_ext._compile() method to change compilee flags
        # at a per-file base. This supposedly doesn't work for MSVC, so
        # we only do it if we are sure that a ._compile() method exists
        #
        # Idea taken from https://stackoverflow.com/a/49772928/18609162
        if not self.compiler_is_msvc():
            original__compile = self.compiler._compile
            if self.with_simd:
                ext.extra_compile_args.extend([f for f in vrna_simd_cflags.values()])

                def new__compile(obj, src, ext, cc_args, extra_postargs, pp_opts):
                    for simd_opt, simd_flag in vrna_simd_cflags.items():
                        if src not in vrna_simd_files[simd_opt]:
                            extra_postargs = [arg for arg in extra_postargs if arg != simd_flag]

                    # remove c++ flags for .c files
                    if src.lower().endswith('.c'):
                      extra_postargs = [arg for arg in extra_postargs if not arg.lower().startswith('-std')]

                    return original__compile(obj, src, ext, cc_args, extra_postargs, pp_opts)
            else:
                def new__compile(obj, src, ext, cc_args, extra_postargs, pp_opts):
                    # remove c++ flags for .c files
                    if src.lower().endswith('.c'):
                      extra_postargs = [arg for arg in extra_postargs if not arg.lower().startswith('-std')]

                    return original__compile(obj, src, ext, cc_args, extra_postargs, pp_opts)

            self.compiler._compile = new__compile

            try:
                build_ext.build_extension(self, ext)
            finally:
                del self.compiler._compile
            return

        return build_ext.build_extension(self, ext)

    def parse_options(self, configfile = "setup.cfg"):
        pyproject_entry = 'build_ext'
        parser = configparser.ConfigParser()
        parser.read(configfile)

        for f in [f.replace('with-','') for f in vrna_flags]:
            switch = 'with_' + f

            # all options default to False
            setattr(self, switch, False)

            if parser.has_option(pyproject_entry, switch):
                setattr(self, switch, parser.getboolean(pyproject_entry, switch))

            if getattr(self, switch):
                # Check if macro name is given
                macro = f + '_macro'
                if parser.has_option(pyproject_entry, macro):
                    m = parser.get(pyproject_entry, macro)
                    if m:
                        setattr(self, macro, m)

                # Check if libraries are given
                libs = f + '_libs'
                if parser.has_option(pyproject_entry, libs):
                    l = [ll for ll in parser.get(pyproject_entry, libs).split()]
                    if l:
                        setattr(self, libs, l)

                # Check compile flags
                cflags = f + '_cflags'
                if parser.has_option(pyproject_entry, cflags):
                    fl = parser.get(pyproject_entry, cflags)
                    if fl:
                        setattr(self, cflags, fl)


def vrna_ext(name = "RNA._RNA"):
    define_macros = []
    libraries = []
    extra_compile_args = []
    extra_link_args = []
    includeDirs = []
    srcFiles = []

    # collect all RNAlib source files
    srcDir = 'src/ViennaRNA'
    globStr = "%s/*.c*" % srcDir
    files = glob.glob(globStr)
    srcFiles += files

    srcFiles += ['src/json/json.c']
    srcFiles += ['src/cephes/kn.c',
                 'src/cephes/const.c',
                 'src/cephes/mtherr.c',
                 'src/cephes/expn.c']

    for root, dirnames, filenames in os.walk(srcDir):
      for dirname in dirnames:
        absPath = os.path.join(root, dirname)
        #print('adding dir to path: %s' % absPath)
        globStr = "%s/*.c*" % absPath
        files = glob.glob(globStr)
        #print(files)
        srcFiles += files

    srcFiles += ['interfaces/Python/RNA_wrap.cpp']

    # append include paths required to build the Python interface
    includeDirs.append('.')
    includeDirs.append('src')
    includeDirs.append('src/ViennaRNA')
    includeDirs.append('src/dlib-20.0')
    includeDirs.append('src/cephes')

    if "src/libsvm-3.35" != "src/":
        srcFiles += ['src/libsvm-3.35/svm.cpp']
        includeDirs.append('src/libsvm-3.35')

    #print("includeDirs:")
    #print(includeDirs)
    #print("srcFiles:")
    #print(srcFiles)

    # comment out every cpp define in config.h that we want to handle
    # from within this setup.py script
    defines = ['#define VRNA_WITH_GSL',
               '#define VRNA_NR_SAMPLING_MPFR',
               '#define VRNA_WITH_SVM',
               '#define VRNA_WITH_NAVIEW_LAYOUT',
               '#define VRNA_WITH_OPENMP',
               '#define VRNA_WITH_SIMD_AVX512',
               '#define VRNA_WITH_SIMD_SSE41',
               '#define VRNA_LOG_NO_DEBUG_RNALIB'
              ]

    comment_lines('config.h', defines)

    return Extension(
        name = name,
        sources = srcFiles,
        include_dirs = includeDirs,
        define_macros = define_macros,
        extra_compile_args = extra_compile_args,
        libraries = libraries,
        language="c++",
        extra_link_args = extra_link_args,
    )


# Build extensions before python modules,
# or the generated SWIG python files will be missing.
#class BuildPy(build_py):
#    def run(self):
#        self.run_command('build_ext')
#        super(build_py, self).run()



setup(
    long_description=LONG_DESCRIPTION,
    long_description_content_type="text/markdown",
    packages=['ViennaRNA', 'RNA'],
    ext_modules=[vrna_ext()],
    cmdclass={'build_ext': vrna_build_ext},
    py_modules = ['RNA.RNA'],
)



import unittest

if __name__ == '__main__':
    from py_include import taprunner
    import RNApath
    RNApath.addSwigInterfacePath()

import RNA

# use this function to avoid problems on platforms
# where qsort yields other orders of elements
def all_shifts(permutations):
    exhaustive_permutations = list()
    n = len(permutations[0])
    for i in range(n):
        for p in permutations:
            exhaustive_permutations.append(p[i:] + p[:i])
    return exhaustive_permutations

class CombinatoricsTests(unittest.TestCase):
    def test_enumerate_necklaces1(self):
        """Enumerate necklaces (4 sequences, 1 strand per sequences)"""
        content   = (1, 1, 1, 1)
        perm_real = ( (0, 3, 2, 1),
                      (0, 3, 1, 2),
                      (0, 2, 3, 1),
                      (0, 2, 1, 3),
                      (0, 1, 3, 2),
                      (0, 1, 2, 3))
        permutations = RNA.enumerate_necklaces(content)
        self.assertEqual(set(all_shifts(perm_real)), set(all_shifts(permutations)))


    def test_enumerate_necklaces2(self):
        """Enumerate necklaces (3 sequences, different number of strands per sequence)"""
        content = (3, 1, 2)
        perm_real = ( (1, 0, 0, 0, 2, 2),
                      (1, 0, 0, 2, 0, 2),
                      (1, 0, 0, 2, 2, 0),
                      (1, 0, 2, 0, 0, 2),
                      (1, 0, 2, 0, 2, 0),
                      (1, 0, 2, 2, 0, 0),
                      (1, 2, 0, 0, 0, 2),
                      (1, 2, 0, 0, 2, 0),
                      (1, 2, 0, 2, 0, 0),
                      (1, 2, 2, 0, 0, 0))
        permutations = RNA.enumerate_necklaces(content)
        self.assertEqual(set(all_shifts(perm_real)), set(all_shifts(permutations)))


if __name__ == '__main__':
    unittest.main(testRunner=taprunner.TAPTestRunner())



import os
import unittest

if __name__ == '__main__':
    from py_include import taprunner
    import RNApath
    RNApath.addSwigInterfacePath()

import RNA


seq_con     = "CCCAAAAGGGCCCAAAAGGG"
str_con     = "..........(((....)))"
str_con_def = "(((....)))(((....)))"

def getShapeDataFromFile(filepath):
    retVec = []
    retVec.append(-999.0);  # data list is 1-based, so we add smth. at pos 0
    count=1
    with open(filepath, 'r') as f:
        lines = f.read().splitlines()

        for line in lines:
            pos = int(line.split(' ')[0])
            value = float(line.split(' ')[1])

            if(pos==count):
                retVec.append(value)
            else:
                for i in range(pos-count):
                    retVec.append(-999.0)
                retVec.append(value)
                count=pos
            count+=1
    return retVec


def getShapeSequenceFromFile(filepath):
    retSeq=""
    with open(filepath,'r') as f:
        lines = f.read().splitlines()

    return lines[0]

class constraintsTest(unittest.TestCase):
    DATADIR = os.environ.get('VRNA_TEST_DATA', os.sep.join(["tests", "data"]))

    def test_sc_add_deigan(self):
        """SHAPE data Deigan et al. 2009 method - Lysine riboswitch"""
        seq          = getShapeSequenceFromFile(os.sep.join([self.DATADIR, "Lysine_riboswitch_T._martima.db"]))
        reactivities = getShapeDataFromFile(os.sep.join([self.DATADIR, "Lysine_riboswitch_T._martima.shape_2rows"]))

        fc=RNA.fold_compound(seq)
        print(reactivities)

        ret = fc.sc_add_SHAPE_deigan(reactivities, 1.9, -0.7, RNA.OPTION_DEFAULT)
        (ss,mfe) = fc.mfe()
        print(ss, "[ %6.4f ]" % mfe)
        self.assertEqual("%6.4f" %mfe,"%6.4f" % -121.55 )


    def test_sc_add_SHAPE_deigan2(self):
        """SHAPE data Deigan et al. 2009 method - TPP riboswitch"""
        seq_ribo     = getShapeSequenceFromFile(os.sep.join([self.DATADIR, "TPP_riboswitch_E.coli.db"]))
        reactivities = getShapeDataFromFile(os.sep.join([self.DATADIR, "TPP_riboswitch_E.coli.shape_2rows"]))

        fc=RNA.fold_compound(seq_ribo)
        print(reactivities)

        ret = fc.sc_add_SHAPE_deigan(reactivities,1.9,-0.7,RNA.OPTION_DEFAULT);  # these values were copied from ronnys Talk about constraints
        (ss,mfe) = fc.mfe()
        print(ss, "[ %6.2f ]" % mfe)
        self.assertEqual("%6.2f" %mfe,"%6.2f" % -52.61 )


    # with randomly choosen sequences and shape data, this test only checks if the function can be called, not if it results correct results.
    def test_sc_add_SHAPE_deigan_ali(self):
        """SHAPE data Deigan et al. 2009 method - Comparative MFE prediction"""
        # shape data from
        shapeSeq1 = "CCCAAAAGGG"
        shapeSeq2 = "AAUAAAAAUU"
        shapeAli = [shapeSeq1,shapeSeq2]

        shapeFiles = [os.sep.join([self.DATADIR, "alignment_1.shape_2rows"]), os.sep.join([self.DATADIR, "alignment_2.shape_2rows"])]
        fc=RNA.fold_compound(shapeAli);
        assoc = [0, 1]
        ret = fc.sc_add_SHAPE_deigan_ali(shapeFiles, assoc,1.8,-0.6)
        (ss,mfe) = fc.mfe()
        print(ss, "[ %6.2f ]" % mfe)
        self.assertEqual(ret,2)


    def test_sc_add_SHAPE_zarringhalam(self):
        """SHAPE data Zarringhalam et al 2012 method"""
        seq_ribo     = getShapeSequenceFromFile(os.sep.join([self.DATADIR, "TPP_riboswitch_E.coli.db"]))
        fc           = RNA.fold_compound(seq_ribo)
        reactivities = getShapeDataFromFile(os.sep.join([self.DATADIR, "TPP_riboswitch_E.coli.shape_2rows"]))

        ret = fc.sc_add_SHAPE_zarringhalam(reactivities, 0.5, 0.5, "O"); # these values were copied from ronnys Talk about constraints, O = default value
        (ss,mfe) = fc.mfe()
        print(ss, "[ %6.2f ]" % mfe)
        self.assertEqual("%6.2f" %mfe,"%6.2f" % -5.28 )


if __name__ == '__main__':
    constraintsTest.DATADIR = RNApath.getDataDirPath()
    unittest.main(testRunner=taprunner.TAPTestRunner())



import unittest

if __name__ == '__main__':
    from py_include import taprunner
    import RNApath
    RNApath.addSwigInterfacePath()

import RNA

seq1          = "CGCAGGGAUACCCGCG"
struct1       = "(((.(((...))))))"
struct1pk     = "(.(.(([....)))])"
struct2       = "(..............)"

class GeneralTests(unittest.TestCase):
    def test_pairtable(self):
        """Create pair table"""
        pairTable = list(RNA.ptable(struct1))
        correctPairTable = [16, 16, 15, 14, 0, 13, 12, 11, 0, 0, 0, 7, 6, 5, 3, 2, 1]
        self.assertEqual(pairTable,correctPairTable)

        pairTable_pk = RNA.ptable_pk(struct1pk)
        correctPairTable_pk = (16, 16, 0, 14, 0, 13, 12, 15, 0, 0, 0, 0, 6, 5, 3, 7, 1)
        self.assertEqual(pairTable_pk,correctPairTable_pk)

        #convert pairtable back to struct1
        self.assertEqual(struct1,RNA.db_from_ptable(pairTable))


    def test_pairtable_slicing(self):
        """Slice pair table"""
        correctPairTable  = [16, 16, 15, 14, 0, 13, 12, 11, 0, 0, 0, 7, 6, 5, 3, 2, 1]
        pt_slice          = RNA.ptable(struct1)[3:12]
        pt_sl_ref         = correctPairTable[3:12]
        self.assertEqual(list(pt_slice), pt_sl_ref)


    def test_basePairDistance(self):
        """Base pair distance"""
        d = RNA.bp_distance("(((.(((...))))))","(((..........)))")
        self.assertEqual(d,3)


    def test_basePairDistancePairTable(self):
        """Base Pair Distance - PairTable"""
        pt1 = RNA.ptable("(((.(((...))))))")
        pt2 = RNA.ptable("(((..........)))")

        d = RNA.bp_distance(pt1,pt2)

        self.assertEqual(d,3)


    def test_basePairDistancePseudoknot(self):
        """Base Pair Distance (pk) - PairTable"""
        d1 = RNA.bp_distance(struct1, struct1pk)
        d2 = RNA.bp_distance(struct1, struct1pk, RNA.BRACKETS_ANY)

        self.assertEqual(d1, 2)
        self.assertEqual(d2, 3)


    def test_plists(self):
        """RNA,plist()"""
        plist = RNA.plist(struct1,0.6)
        print(plist)


    def test_filename_sanitize_simple(self):
        """Sanitize file names - with directories and special characters"""
        fn = "bla/bla??_foo\\bar\"r<u>m:ble"
        fs = RNA.filename_sanitize(fn)
        self.assertEqual(fs, "blabla_foobarrumble")
        fs = RNA.filename_sanitize(fn, '-')
        self.assertEqual(fs, "bla-bla--_foo-bar-r-u-m-ble")


    def test_filename_sanitize_special(self):
        """Sanitize file names - special characters"""
        fn = "??"
        fs = RNA.filename_sanitize(fn)
        self.assertEqual(fs, "")
        fs = RNA.filename_sanitize(fn, '.')
        self.assertEqual(fs, "")


    def test_filename_sanitize_long(self):
        """Sanitize (too) long file names"""
        fn = "%s%s%sDEFGHIJ.svg" % ("A" * 120, "B" * 120, "C" * 10)
        fs = RNA.filename_sanitize(fn)
        self.assertEqual(fs, "%s%s%sD.svg" % ("A" * 120, "B" * 120, "C" * 10))
        fn = "A.%s%s%sDEFGHIsvg" % ("A" * 120, "B" * 120, "C" * 10)
        fs = RNA.filename_sanitize(fn)
        self.assertEqual(fs, "A.%s%s%sDEF" % ("A" * 120, "B" * 120, "C" * 10))



if __name__ == '__main__':
    unittest.main(testRunner=taprunner.TAPTestRunner())
